# Azle/Motoko/Rust Benchmarks

-   These benchmarks should be considered preliminary (especially the Motoko benchmarks, something seems off with the function prelude)
-   These benchmarks were implemented using the performance counter API
    -   Performance counter information in [The Internet Computer Interface Spec](https://internetcomputer.org/docs/current/references/ic-interface-spec/#system-api-imports)
    -   Performance counter information in [the forum](https://forum.dfinity.org/t/introducing-performance-counter-on-the-internet-computer/14027)
-   Each benchmark is the average of 10 runs
-   The following may be inaccurate or missing from the benchmarks as described [here](https://forum.dfinity.org/t/introducing-performance-counter-on-the-internet-computer/14027):
    -   Candid serialization/deserialization of function parameters and return types
    -   Canister method prologue/epilogue
    -   Some Motoko runtime behavior (such as garbage collection)
-   You can find a raw CSV file with this data [here](./benchmarks.csv)
-   A rough USD cost model for various app scenarios can be found [here](https://docs.google.com/spreadsheets/d/1PQ53R9hYE1fuMB_z-Bl6dyymm7end7rVJ85TvGEh0BQ)

The format for benchmark numbers is (x / y) where:

-   x = Wasm instructions counted only in the function body
-   y = Wasm instructions counted in the function body and the function prelude

## USD Cost Estimates Per Year

These estimates use the average Wasm instructions per update function call including the function prelude from the benchmarks below.

The Wasm instruction counts used are:

-   Azle: 128_073_623
-   Motoko: 326_842
-   Rust: 135_153

### Application Scenarios

| Usage    | Query/Update Heaviness | Ingress Bytes Per Query Message | Ingress Bytes Per Update Message | GB Storage | Query Messages Per Second | Update Messages Per Second | Xnet Calls Per Second | Xnet Call Bytes |
| -------- | ---------------------- | ------------------------------- | -------------------------------- | ---------- | ------------------------- | -------------------------- | --------------------- | --------------- |
| Light    | Even                   | 100                             | 100                              | 0.5        | 0.01                      | 0.01                       | 0.001                 | 20              |
| Light    | Query Heavy            | 100                             | 100                              | 0.5        | 0.01                      | 0.0001                     | 0.001                 | 20              |
| Light    | Update Heavy           | 100                             | 100                              | 0.5        | 0.0001                    | 0.01                       | 0.001                 | 20              |
| Moderate | Even                   | 1_000                           | 1_000                            | 1          | 1                         | 1                          | 0.1                   | 200             |
| Moderate | Query Heavy            | 1_000                           | 1_000                            | 1          | 1                         | 0.01                       | 0.1                   | 200             |
| Moderate | Update Heavy           | 1_000                           | 1_000                            | 1          | 0.01                      | 1                          | 0.1                   | 200             |
| Heavy    | Even                   | 10_000                          | 10_000                           | 2          | 100                       | 100                        | 10                    | 2_000           |
| Heavy    | Query Heavy            | 10_000                          | 10_000                           | 2          | 100                       | 1                          | 10                    | 2_000           |
| Heavy    | Update Heavy           | 10_000                          | 10_000                           | 2          | 1                         | 100                        | 10                    | 2_000           |

### Application USD Cost Estimates Per Year

| Usage    | Query/Update Heaviness | CDK    | Ingress Messages | Ingress Bytes Query Messages | Ingress Bytes Update Messages | Update Messages | Update Instructions | Xnet Calls | Xnet Byte Transmission | GB Storage | Total Cost  |
| -------- | ---------------------- | ------ | ---------------- | ---------------------------- | ----------------------------- | --------------- | ------------------- | ---------- | ---------------------- | ---------- | ----------- |
| Light    | Even                   | Azle   | $1.00            | $0.08                        | $0.08                         | $0.25           | $21.33              | $0.01      | $0.00                  | $2.64      | $25.39      |
| Light    | Even                   | Motoko | $1.00            | $0.08                        | $0.08                         | $0.25           | $0.05               | $0.01      | $0.00                  | $2.64      | $4.12       |
| Light    | Even                   | Rust   | $1.00            | $0.08                        | $0.08                         | $0.25           | $0.02               | $0.01      | $0.00                  | $2.64      | $4.09       |
| Light    | Query Heavy            | Azle   | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.21               | $0.01      | $0.00                  | $2.64      | $3.46       |
| Light    | Query Heavy            | Motoko | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.00               | $0.01      | $0.00                  | $2.64      | $3.25       |
| Light    | Query Heavy            | Rust   | $0.50            | $0.08                        | $0.00                         | $0.00           | $0.00               | $0.01      | $0.00                  | $2.64      | $3.25       |
| Light    | Update Heavy           | Azle   | $0.50            | $0.00                        | $0.08                         | $0.25           | $21.33              | $0.01      | $0.00                  | $2.64      | $24.81      |
| Light    | Update Heavy           | Motoko | $0.50            | $0.00                        | $0.08                         | $0.25           | $0.05               | $0.01      | $0.00                  | $2.64      | $3.54       |
| Light    | Update Heavy           | Rust   | $0.50            | $0.00                        | $0.08                         | $0.25           | $0.02               | $0.01      | $0.00                  | $2.64      | $3.51       |
| Moderate | Even                   | Azle   | $99.91           | $83.26                       | $83.26                        | $24.56          | $2,132.55           | $1.08      | $0.83                  | $5.29      | $2,430.73   |
| Moderate | Even                   | Motoko | $99.91           | $83.26                       | $83.26                        | $24.56          | $5.44               | $1.08      | $0.83                  | $5.29      | $303.62     |
| Moderate | Even                   | Rust   | $99.91           | $83.26                       | $83.26                        | $24.56          | $2.25               | $1.08      | $0.83                  | $5.29      | $300.43     |
| Moderate | Query Heavy            | Azle   | $50.45           | $83.26                       | $0.83                         | $0.25           | $21.33              | $1.08      | $0.83                  | $5.29      | $163.31     |
| Moderate | Query Heavy            | Motoko | $50.45           | $83.26                       | $0.83                         | $0.25           | $0.05               | $1.08      | $0.83                  | $5.29      | $142.04     |
| Moderate | Query Heavy            | Rust   | $50.45           | $83.26                       | $0.83                         | $0.25           | $0.02               | $1.08      | $0.83                  | $5.29      | $142.01     |
| Moderate | Update Heavy           | Azle   | $50.45           | $0.83                        | $83.26                        | $24.56          | $2,132.55           | $1.08      | $0.83                  | $5.29      | $2,298.86   |
| Moderate | Update Heavy           | Motoko | $50.45           | $0.83                        | $83.26                        | $24.56          | $5.44               | $1.08      | $0.83                  | $5.29      | $171.74     |
| Moderate | Update Heavy           | Rust   | $50.45           | $0.83                        | $83.26                        | $24.56          | $2.25               | $1.08      | $0.83                  | $5.29      | $168.55     |
| Heavy    | Even                   | Azle   | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $213,255.49         | $108.23    | $832.55                | $10.57     | $393,163.56 |
| Heavy    | Even                   | Motoko | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $544.23             | $108.23    | $832.55                | $10.57     | $180,452.29 |
| Heavy    | Even                   | Rust   | $9,990.60        | $83,255.04                   | $83,255.04                    | $2,456.02       | $225.04             | $108.23    | $832.55                | $10.57     | $180,133.11 |
| Heavy    | Query Heavy            | Azle   | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $2,132.55           | $108.23    | $832.55                | $10.57     | $92,241.32  |
| Heavy    | Query Heavy            | Motoko | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $5.44               | $108.23    | $832.55                | $10.57     | $90,114.20  |
| Heavy    | Query Heavy            | Rust   | $5,045.26        | $83,255.04                   | $832.55                       | $24.56          | $2.25               | $108.23    | $832.55                | $10.57     | $90,111.01  |
| Heavy    | Update Heavy           | Azle   | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $213,255.49         | $108.23    | $832.55                | $10.57     | $305,795.72 |
| Heavy    | Update Heavy           | Motoko | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $544.23             | $108.23    | $832.55                | $10.57     | $93,084.45  |
| Heavy    | Update Heavy           | Rust   | $5,045.26        | $832.55                      | $83,255.04                    | $2,456.02       | $225.04             | $108.23    | $832.55                | $10.57     | $92,765.27  |

## Averages

| Azle Wasm Instructions      | Motoko Wasm Instructions | Rust Wasm Instructions | Azle/Motoko Change Multiplier | Azle/Rust Change Multiplier | Motoko/Azle Change Multiplier | Motoko/Rust Change Multiplier | Rust/Azle Change Multiplier | Rust/Motoko Change Multiplier |
| --------------------------- | ------------------------ | ---------------------- | ----------------------------- | --------------------------- | ----------------------------- | ----------------------------- | --------------------------- | ----------------------------- |
| (127_649_854 / 128_073_623) | (325_462 / 326_842)      | (101_345 / 135_153)    | (2_063x / 1_386x)             | (2_376x / 272x)             | (-2_063x / -1_386x)           | (1x / -8x)                    | (-2_376x / -272x)           | (-1x / 8x)                    |

## Benchmarks

| Description               | Azle Wasm Instructions          | Motoko Wasm Instructions | Rust Wasm Instructions  | Azle/Motoko Change Multiplier | Azle/Rust Change Multiplier | Motoko/Azle Change Multiplier | Motoko/Rust Change Multiplier | Rust/Azle Change Multiplier | Rust/Motoko Change Multiplier |
| ------------------------- | ------------------------------- | ------------------------ | ----------------------- | ----------------------------- | --------------------------- | ----------------------------- | ----------------------------- | --------------------------- | ----------------------------- |
| empty                     | (63_757 / 432_497)              | (200 / 1_264)            | (200 / 19_466)          | (319x / 342x)                 | (319x / 22x)                | (-319x / -342x)               | (1x / -15x)                   | (-319x / -22x)              | (1x / 15x)                    |
| blob_init_stack: 1        | (1_375_743 / 1_783_455)         | (646 / 2_029)            | (1_030 / 36_624)        | (2_130x / 879x)               | (1_336x / 49x)              | (-2_130x / -879x)             | (-2x / -18x)                  | (-1_336x / -49x)            | (2x / 18x)                    |
| blob_init_stack: 10       | (8_594_091 / 9_001_838)         | (2_866 / 4_249)          | (4_529 / 39_983)        | (2_999x / 2_119x)             | (1_898x / 225x)             | (-2_999x / -2_119x)           | (-2x / -9x)                   | (-1_898x / -225x)           | (2x / 9x)                     |
| blob_init_stack: 100      | (84_371_843 / 84_779_282)       | (26_716 / 28_099)        | (43_454 / 78_908)       | (3_158x / 3_017x)             | (1_942x / 1_074x)           | (-3_158x / -3_017x)           | (-2x / -3x)                   | (-1_942x / -1_074x)         | (2x / 3x)                     |
| blob_init_heap: 1         | (1_444_917 / 1_851_723)         | (3_756 / 5_139)          | (3_117 / 38_631)        | (385x / 361x)                 | (470x / 48x)                | (-385x / -361x)               | (1x / -8x)                    | (-470x / -48x)              | (-1x / 8x)                    |
| blob_init_heap: 10        | (8_989_567 / 9_722_733)         | (35_526 / 36_909)        | (24_871 / 59_395)       | (253x / 264x)                 | (366x / 164x)               | (-253x / -264x)               | (1x / -2x)                    | (-366x / -164x)             | (-1x / 2x)                    |
| blob_init_heap: 100       | (92_601_891 / 93_007_569)       | (1_858_219 / 1_859_602)  | (240_023 / 273_869)     | (51x / 51x)                   | (392x / 344x)               | (-51x / -51x)                 | (8x / 7x)                     | (-392x / -344x)             | (-8x / -7x)                   |
| boolean_init_stack: 1     | (332_028 / 739_386)             | (251 / 1_634)            | (219 / 33_911)          | (1_323x / 453x)               | (1_516x / 22x)              | (-1_323x / -453x)             | (1x / -21x)                   | (-1_516x / -22x)            | (-1x / 21x)                   |
| boolean_init_stack: 10    | (1_976_652 / 2_383_477)         | (566 / 1_949)            | (363 / 34_055)          | (3_492x / 1_223x)             | (5_445x / 70x)              | (-3_492x / -1_223x)           | (2x / -17x)                   | (-5_445x / -70x)            | (-2x / 17x)                   |
| boolean_init_stack: 100   | (18_867_405 / 19_273_282)       | (3_716 / 5_099)          | (1_803 / 35_495)        | (5_077x / 3_780x)             | (10_464x / 543x)            | (-5_077x / -3_780x)           | (2x / -7x)                    | (-10_464x / -543x)          | (-2x / 7x)                    |
| boolean_init_heap: 1      | (401_121 / 808_093)             | (3_361 / 4_744)          | (2_348 / 36_914)        | (120x / 171x)                 | (174x / 22x)                | (-120x / -171x)               | (1x / -8x)                    | (-174x / -22x)              | (-1x / 8x)                    |
| boolean_init_heap: 10     | (2_669_321 / 3_077_174)         | (33_081 / 34_464)        | (21_319 / 54_712)       | (81x / 89x)                   | (127x / 56x)                | (-81x / -89x)                 | (1x / -2x)                    | (-127x / -56x)              | (-1x / 2x)                    |
| boolean_init_heap: 100    | (26_517_251 / 26_922_493)       | (1_833_763 / 1_835_146)  | (215_596 / 249_290)     | (15x / 15x)                   | (125x / 109x)               | (-15x / -15x)                 | (8x / 7x)                     | (-125x / -109x)             | (-8x / -7x)                   |
| float32_init_stack: 1     | (345_839 / 751_542)             | (252 / 1_635)            | (219 / 33_941)          | (1_372x / 460x)               | (1_579x / 22x)              | (-1_372x / -460x)             | (1x / -21x)                   | (-1_579x / -22x)            | (-1x / 21x)                   |
| float32_init_stack: 10    | (2_116_060 / 2_523_934)         | (576 / 1_959)            | (363 / 34_085)          | (3_674x / 1_288x)             | (5_829x / 74x)              | (-3_674x / -1_288x)           | (2x / -17x)                   | (-5_829x / -74x)            | (-2x / 17x)                   |
| float32_init_stack: 100   | (20_278_158 / 20_686_391)       | (3_816 / 5_199)          | (1_803 / 35_525)        | (5_314x / 3_979x)             | (11_247x / 582x)            | (-5_314x / -3_979x)           | (2x / -7x)                    | (-11_247x / -582x)          | (-2x / 7x)                    |
| float32_init_heap: 1      | (413_014 / 814_813)             | (3_414 / 4_797)          | (2_778 / 36_504)        | (121x / 170x)                 | (150x / 22x)                | (-121x / -170x)               | (1x / -8x)                    | (-150x / -22x)              | (-1x / 8x)                    |
| float32_init_heap: 10     | (2_799_103 / 3_203_211)         | (33_756 / 35_139)        | (25_665 / 58_964)       | (83x / 91x)                   | (110x / 54x)                | (-83x / -91x)                 | (1x / -2x)                    | (-110x / -54x)              | (-1x / 2x)                    |
| float32_init_heap: 100    | (27_485_185 / 27_892_479)       | (1_840_518 / 1_841_901)  | (259_127 / 292_821)     | (15x / 16x)                   | (106x / 96x)                | (-15x / -16x)                 | (7x / 6x)                     | (-106x / -96x)              | (-7x / -6x)                   |
| float64_init_stack: 1     | (346_154 / 755_271)             | (252 / 1_635)            | (217 / 33_939)          | (1_374x / 462x)               | (1_595x / 22x)              | (-1_374x / -462x)             | (1x / -21x)                   | (-1_595x / -22x)            | (-1x / 21x)                   |
| float64_init_stack: 10    | (2_117_549 / 2_526_906)         | (576 / 1_959)            | (343 / 34_065)          | (3_676x / 1_290x)             | (6_174x / 74x)              | (-3_676x / -1_290x)           | (2x / -17x)                   | (-6_174x / -74x)            | (-2x / 17x)                   |
| float64_init_stack: 100   | (20_239_491 / 20_644_921)       | (3_816 / 5_199)          | (1_603 / 35_325)        | (5_304x / 3_971x)             | (12_626x / 584x)            | (-5_304x / -3_971x)           | (2x / -7x)                    | (-12_626x / -584x)          | (-2x / 7x)                    |
| float64_init_heap: 1      | (414_279 / 821_619)             | (3_358 / 4_741)          | (2_398 / 35_717)        | (123x / 173x)                 | (178x / 23x)                | (-123x / -173x)               | (1x / -8x)                    | (-178x / -23x)              | (-1x / 8x)                    |
| float64_init_heap: 10     | (2_802_279 / 3_210_335)         | (34_129 / 35_512)        | (25_694 / 58_584)       | (82x / 90x)                   | (110x / 55x)                | (-82x / -90x)                 | (1x / -2x)                    | (-110x / -55x)              | (-1x / 2x)                    |
| float64_init_heap: 100    | (27_854_000 / 28_258_596)       | (1_716_211 / 1_717_594)  | (260_300 / 293_565)     | (16x / 16x)                   | (107x / 97x)                | (-16x / -16x)                 | (7x / 6x)                     | (-107x / -97x)              | (-7x / -6x)                   |
| int_init_stack: 1         | (332_122 / 738_443)             | (251 / 1_634)            | (1_419 / 34_710)        | (1_323x / 452x)               | (234x / 21x)                | (-1_323x / -452x)             | (-6x / -21x)                  | (-234x / -21x)              | (6x / 21x)                    |
| int_init_stack: 10        | (1_980_720 / 2_388_834)         | (566 / 1_949)            | (6_454 / 39_745)        | (3_500x / 1_226x)             | (307x / 60x)                | (-3_500x / -1_226x)           | (-11x / -20x)                 | (-307x / -60x)              | (11x / 20x)                   |
| int_init_stack: 100       | (18_914_996 / 19_323_417)       | (3_716 / 5_099)          | (62_704 / 95_995)       | (5_090x / 3_790x)             | (302x / 201x)               | (-5_090x / -3_790x)           | (-17x / -19x)                 | (-302x / -201x)             | (17x / 19x)                   |
| int_init_heap: 1          | (400_072 / 801_789)             | (3_361 / 4_744)          | (3_700 / 36_785)        | (119x / 169x)                 | (110x / 22x)                | (-119x / -169x)               | (-1x / -8x)                   | (-110x / -22x)              | (1x / 8x)                     |
| int_init_heap: 10         | (2_669_156 / 3_073_748)         | (33_081 / 34_464)        | (27_452 / 60_298)       | (81x / 89x)                   | (98x / 51x)                 | (-81x / -89x)                 | (1x / -2x)                    | (-98x / -51x)               | (-1x / 2x)                    |
| int_init_heap: 100        | (26_161_834 / 26_569_360)       | (1_833_763 / 1_835_146)  | (263_432 / 296_552)     | (15x / 15x)                   | (101x / 91x)                | (-15x / -15x)                 | (7x / 6x)                     | (-101x / -91x)              | (-7x / -6x)                   |
| int8_init_stack: 1        | (332_481 / 741_123)             | (251 / 1_634)            | (219 / 33_672)          | (1_325x / 454x)               | (1_518x / 22x)              | (-1_325x / -454x)             | (1x / -21x)                   | (-1_518x / -22x)            | (-1x / 21x)                   |
| int8_init_stack: 10       | (2_354_688 / 2_761_157)         | (566 / 1_949)            | (363 / 33_816)          | (4_160x / 1_417x)             | (6_487x / 82x)              | (-4_160x / -1_417x)           | (2x / -17x)                   | (-6_487x / -82x)            | (-2x / 17x)                   |
| int8_init_stack: 100      | (18_484_343 / 18_890_297)       | (3_716 / 5_099)          | (1_803 / 35_256)        | (4_974x / 3_705x)             | (10_252x / 536x)            | (-4_974x / -3_705x)           | (2x / -7x)                    | (-10_252x / -536x)          | (-2x / 7x)                    |
| int8_init_heap: 1         | (401_407 / 810_650)             | (3_332 / 4_715)          | (2_352 / 35_468)        | (121x / 172x)                 | (174x / 23x)                | (-121x / -172x)               | (1x / -8x)                    | (-174x / -23x)              | (-1x / 8x)                    |
| int8_init_heap: 10        | (2_673_615 / 3_083_368)         | (32_935 / 34_318)        | (21_307 / 54_311)       | (81x / 90x)                   | (128x / 57x)                | (-81x / -90x)                 | (1x / -2x)                    | (-128x / -57x)              | (-1x / 2x)                    |
| int8_init_heap: 100       | (26_532_894 / 26_937_755)       | (1_832_308 / 1_833_691)  | (258_614 / 292_282)     | (15x / 15x)                   | (103x / 92x)                | (-15x / -15x)                 | (7x / 6x)                     | (-103x / -92x)              | (-7x / -6x)                   |
| int16_init_stack: 1       | (332_287 / 737_998)             | (251 / 1_634)            | (219 / 33_941)          | (1_324x / 452x)               | (1_517x / 22x)              | (-1_324x / -452x)             | (1x / -21x)                   | (-1_517x / -22x)            | (-1x / 21x)                   |
| int16_init_stack: 10      | (1_978_778 / 2_387_003)         | (566 / 1_949)            | (363 / 34_085)          | (3_496x / 1_225x)             | (5_451x / 70x)              | (-3_496x / -1_225x)           | (2x / -17x)                   | (-5_451x / -70x)            | (-2x / 17x)                   |
| int16_init_stack: 100     | (18_877_851 / 19_285_583)       | (3_716 / 5_099)          | (1_803 / 35_525)        | (5_080x / 3_782x)             | (10_470x / 543x)            | (-5_080x / -3_782x)           | (2x / -7x)                    | (-10_470x / -543x)          | (-2x / 7x)                    |
| int16_init_heap: 1        | (399_750 / 801_595)             | (3_332 / 4_715)          | (2_820 / 36_563)        | (120x / 170x)                 | (144x / 22x)                | (-120x / -170x)               | (1x / -8x)                    | (-144x / -22x)              | (-1x / 8x)                    |
| int16_init_heap: 10       | (2_669_130 / 3_074_849)         | (32_935 / 34_318)        | (25_363 / 58_270)       | (81x / 90x)                   | (106x / 53x)                | (-81x / -90x)                 | (1x / -2x)                    | (-106x / -53x)              | (-1x / 2x)                    |
| int16_init_heap: 100      | (26_173_816 / 26_581_655)       | (1_832_308 / 1_833_691)  | (260_596 / 294_222)     | (15x / 15x)                   | (101x / 91x)                | (-15x / -15x)                 | (7x / 6x)                     | (-101x / -91x)              | (-7x / -6x)                   |
| int32_init_stack: 1       | (332_714 / 1_114_725)           | (258 / 1_641)            | (219 / 33_914)          | (1_290x / 679x)               | (1_519x / 33x)              | (-1_290x / -679x)             | (1x / -21x)                   | (-1_519x / -33x)            | (-1x / 21x)                   |
| int32_init_stack: 10      | (1_974_718 / 2_375_894)         | (641 / 2_024)            | (363 / 34_058)          | (3_081x / 1_174x)             | (5_440x / 70x)              | (-3_081x / -1_174x)           | (2x / -17x)                   | (-5_440x / -70x)            | (-2x / 17x)                   |
| int32_init_stack: 100     | (18_485_133 / 18_890_773)       | (4_466 / 5_849)          | (1_803 / 35_498)        | (4_139x / 3_230x)             | (10_252x / 532x)            | (-4_139x / -3_230x)           | (2x / -6x)                    | (-10_252x / -532x)          | (-2x / 6x)                    |
| int32_init_heap: 1        | (401_407 / 810_147)             | (3_424 / 4_807)          | (2_326 / 35_642)        | (117x / 169x)                 | (175x / 23x)                | (-117x / -169x)               | (1x / -7x)                    | (-175x / -23x)              | (-1x / 7x)                    |
| int32_init_heap: 10       | (2_676_487 / 3_085_155)         | (33_476 / 34_859)        | (21_360 / 55_040)       | (80x / 89x)                   | (127x / 56x)                | (-80x / -89x)                 | (1x / -2x)                    | (-127x / -56x)              | (-1x / 2x)                    |
| int32_init_heap: 100      | (26_524_535 / 26_927_871)       | (1_837_713 / 1_839_096)  | (216_295 / 250_037)     | (15x / 15x)                   | (124x / 109x)               | (-15x / -15x)                 | (8x / 7x)                     | (-124x / -109x)             | (-8x / -7x)                   |
| int64_init_stack: 1       | (332_286 / 737_798)             | (258 / 1_641)            | (217 / 33_939)          | (1_288x / 450x)               | (1_531x / 22x)              | (-1_288x / -450x)             | (1x / -21x)                   | (-1_531x / -22x)            | (-1x / 21x)                   |
| int64_init_stack: 10      | (1_981_250 / 2_390_013)         | (646 / 2_029)            | (343 / 34_677)          | (3_067x / 1_178x)             | (5_776x / 69x)              | (-3_067x / -1_178x)           | (2x / -17x)                   | (-5_776x / -69x)            | (-2x / 17x)                   |
| int64_init_stack: 100     | (18_887_835 / 19_293_813)       | (4_516 / 5_899)          | (1_603 / 35_937)        | (4_182x / 3_271x)             | (11_783x / 537x)            | (-4_182x / -3_271x)           | (3x / -6x)                    | (-11_783x / -537x)          | (-3x / 6x)                    |
| int64_init_heap: 1        | (400_060 / 802_576)             | (3_431 / 4_814)          | (2_355 / 36_286)        | (117x / 167x)                 | (173x / 22x)                | (-117x / -167x)               | (1x / -8x)                    | (-173x / -22x)              | (-1x / 8x)                    |
| int64_init_heap: 10       | (2_671_282 / 3_078_297)         | (33_556 / 34_939)        | (25_670 / 59_190)       | (80x / 88x)                   | (105x / 52x)                | (-80x / -88x)                 | (1x / -2x)                    | (-105x / -52x)              | (-1x / 2x)                    |
| int64_init_heap: 100      | (26_556_390 / 26_965_328)       | (1_838_513 / 1_839_896)  | (259_711 / 293_590)     | (15x / 15x)                   | (103x / 92x)                | (-15x / -15x)                 | (7x / 6x)                     | (-103x / -92x)              | (-7x / -6x)                   |
| nat_init_stack: 1         | (331_652 / 733_522)             | (251 / 1_634)            | (1_411 / 35_314)        | (1_321x / 449x)               | (235x / 21x)                | (-1_321x / -449x)             | (-6x / -22x)                  | (-235x / -21x)              | (6x / 22x)                    |
| nat_init_stack: 10        | (1_976_814 / 2_381_046)         | (566 / 1_949)            | (6_529 / 40_432)        | (3_493x / 1_222x)             | (303x / 59x)                | (-3_493x / -1_222x)           | (-12x / -21x)                 | (-303x / -59x)              | (12x / 21x)                   |
| nat_init_stack: 100       | (18_504_980 / 18_912_027)       | (3_716 / 5_099)          | (63_454 / 97_357)       | (4_980x / 3_709x)             | (292x / 194x)               | (-4_980x / -3_709x)           | (-17x / -19x)                 | (-292x / -194x)             | (17x / 19x)                   |
| nat_init_heap: 1          | (401_381 / 810_030)             | (3_361 / 4_744)          | (3_440 / 37_066)        | (120x / 171x)                 | (119x / 22x)                | (-120x / -171x)               | (-0x / -8x)                   | (-119x / -22x)              | (0x / 8x)                     |
| nat_init_heap: 10         | (2_681_264 / 3_091_360)         | (33_081 / 34_464)        | (26_709 / 60_334)       | (81x / 90x)                   | (102x / 51x)                | (-81x / -90x)                 | (1x / -2x)                    | (-102x / -51x)              | (-1x / 2x)                    |
| nat_init_heap: 100        | (26_557_098 / 26_960_368)       | (1_833_763 / 1_835_146)  | (260_021 / 293_788)     | (15x / 15x)                   | (103x / 92x)                | (-15x / -15x)                 | (7x / 6x)                     | (-103x / -92x)              | (-7x / -6x)                   |
| nat8_init_stack: 1        | (331_676 / 738_738)             | (251 / 1_634)            | (219 / 33_618)          | (1_321x / 452x)               | (1_515x / 22x)              | (-1_321x / -452x)             | (1x / -21x)                   | (-1_515x / -22x)            | (-1x / 21x)                   |
| nat8_init_stack: 10       | (1_978_661 / 2_387_642)         | (566 / 1_949)            | (363 / 33_762)          | (3_496x / 1_225x)             | (5_451x / 71x)              | (-3_496x / -1_225x)           | (2x / -17x)                   | (-5_451x / -71x)            | (-2x / 17x)                   |
| nat8_init_stack: 100      | (18_853_553 / 19_258_849)       | (3_716 / 5_099)          | (1_803 / 35_202)        | (5_074x / 3_777x)             | (10_457x / 547x)            | (-5_074x / -3_777x)           | (2x / -7x)                    | (-10_457x / -547x)          | (-2x / 7x)                    |
| nat8_init_heap: 1         | (399_691 / 802_929)             | (3_332 / 4_715)          | (2_318 / 36_131)        | (120x / 170x)                 | (174x / 22x)                | (-120x / -170x)               | (1x / -8x)                    | (-174x / -22x)              | (-1x / 8x)                    |
| nat8_init_heap: 10        | (2_669_866 / 3_076_977)         | (32_935 / 34_318)        | (21_208 / 54_663)       | (81x / 90x)                   | (128x / 56x)                | (-81x / -90x)                 | (1x / -2x)                    | (-128x / -56x)              | (-1x / 2x)                    |
| nat8_init_heap: 100       | (26_518_165 / 26_925_843)       | (1_832_308 / 1_833_691)  | (258_952 / 292_764)     | (15x / 15x)                   | (103x / 92x)                | (-15x / -15x)                 | (7x / 6x)                     | (-103x / -92x)              | (-7x / -6x)                   |
| nat16_init_stack: 1       | (331_555 / 733_696)             | (251 / 1_634)            | (219 / 34_055)          | (1_321x / 449x)               | (1_514x / 22x)              | (-1_321x / -449x)             | (1x / -21x)                   | (-1_514x / -22x)            | (-1x / 21x)                   |
| nat16_init_stack: 10      | (1_974_710 / 2_379_157)         | (566 / 1_949)            | (363 / 34_199)          | (3_489x / 1_221x)             | (5_440x / 70x)              | (-3_489x / -1_221x)           | (2x / -18x)                   | (-5_440x / -70x)            | (-2x / 18x)                   |
| nat16_init_stack: 100     | (18_484_246 / 18_891_701)       | (3_716 / 5_099)          | (1_803 / 35_639)        | (4_974x / 3_705x)             | (10_252x / 530x)            | (-4_974x / -3_705x)           | (2x / -7x)                    | (-10_252x / -530x)          | (-2x / 7x)                    |
| nat16_init_heap: 1        | (401_432 / 811_003)             | (3_332 / 4_715)          | (2_693 / 36_620)        | (121x / 172x)                 | (150x / 22x)                | (-121x / -172x)               | (1x / -8x)                    | (-150x / -22x)              | (-1x / 8x)                    |
| nat16_init_heap: 10       | (2_675_377 / 3_483_612)         | (32_935 / 34_318)        | (25_599 / 58_730)       | (81x / 101x)                  | (105x / 59x)                | (-81x / -101x)                | (1x / -2x)                    | (-105x / -59x)              | (-1x / 2x)                    |
| nat16_init_heap: 100      | (26_123_067 / 26_525_386)       | (1_832_308 / 1_833_691)  | (258_324 / 292_524)     | (15x / 15x)                   | (102x / 91x)                | (-15x / -15x)                 | (7x / 6x)                     | (-102x / -91x)              | (-7x / -6x)                   |
| nat32_init_stack: 1       | (332_203 / 739_880)             | (259 / 1_642)            | (219 / 34_526)          | (1_283x / 451x)               | (1_517x / 21x)              | (-1_283x / -451x)             | (1x / -21x)                   | (-1_517x / -21x)            | (-1x / 21x)                   |
| nat32_init_stack: 10      | (1_978_699 / 2_388_044)         | (646 / 2_029)            | (363 / 34_670)          | (3_063x / 1_177x)             | (5_451x / 69x)              | (-3_063x / -1_177x)           | (2x / -17x)                   | (-5_451x / -69x)            | (-2x / 17x)                   |
| nat32_init_stack: 100     | (18_845_231 / 19_249_938)       | (4_516 / 5_899)          | (1_803 / 36_110)        | (4_173x / 3_263x)             | (10_452x / 533x)            | (-4_173x / -3_263x)           | (3x / -6x)                    | (-10_452x / -533x)          | (-3x / 6x)                    |
| nat32_init_heap: 1        | (400_341 / 805_237)             | (3_425 / 4_808)          | (2_323 / 36_251)        | (117x / 168x)                 | (175x / 22x)                | (-117x / -168x)               | (1x / -8x)                    | (-175x / -22x)              | (-1x / 8x)                    |
| nat32_init_heap: 10       | (2_672_819 / 3_080_754)         | (33_481 / 34_864)        | (21_568 / 55_860)       | (80x / 88x)                   | (126x / 55x)                | (-80x / -88x)                 | (1x / -2x)                    | (-126x / -55x)              | (-1x / 2x)                    |
| nat32_init_heap: 100      | (26_531_798 / 26_939_499)       | (1_837_763 / 1_839_146)  | (216_294 / 250_648)     | (15x / 15x)                   | (124x / 109x)               | (-15x / -15x)                 | (8x / 7x)                     | (-124x / -109x)             | (-8x / -7x)                   |
| nat64_init_stack: 1       | (331_832 / 734_558)             | (260 / 1_643)            | (217 / 34_551)          | (1_276x / 447x)               | (1_529x / 21x)              | (-1_276x / -447x)             | (1x / -21x)                   | (-1_529x / -21x)            | (-1x / 21x)                   |
| nat64_init_stack: 10      | (1_977_033 / 2_381_770)         | (656 / 2_039)            | (343 / 34_677)          | (3_014x / 1_168x)             | (5_764x / 69x)              | (-3_014x / -1_168x)           | (2x / -17x)                   | (-5_764x / -69x)            | (-2x / 17x)                   |
| nat64_init_stack: 100     | (18_508_631 / 18_915_942)       | (4_616 / 5_999)          | (1_603 / 35_937)        | (4_010x / 3_153x)             | (11_546x / 526x)            | (-4_010x / -3_153x)           | (3x / -6x)                    | (-11_546x / -526x)          | (-3x / 6x)                    |
| nat64_init_heap: 1        | (401_908 / 811_224)             | (3_357 / 4_740)          | (2_353 / 36_284)        | (120x / 171x)                 | (174x / 22x)                | (-120x / -171x)               | (1x / -8x)                    | (-174x / -22x)              | (-1x / 8x)                    |
| nat64_init_heap: 10       | (3_071_870 / 3_475_478)         | (33_185 / 34_568)        | (25_650 / 59_170)       | (94x / 102x)                  | (117x / 58x)                | (-94x / -102x)                | (1x / -2x)                    | (-117x / -58x)              | (-1x / 2x)                    |
| nat64_init_heap: 100      | (26_152_479 / 26_556_186)       | (1_834_808 / 1_836_191)  | (259_353 / 293_233)     | (15x / 15x)                   | (101x / 91x)                | (-15x / -15x)                 | (7x / 6x)                     | (-101x / -91x)              | (-7x / -6x)                   |
| null_init_stack: 1        | (332_383 / 740_799)             | (251 / 1_634)            | (219 / 34_122)          | (1_324x / 453x)               | (1_518x / 22x)              | (-1_324x / -453x)             | (1x / -21x)                   | (-1_518x / -22x)            | (-1x / 21x)                   |
| null_init_stack: 10       | (1_979_300 / 2_388_921)         | (566 / 1_949)            | (363 / 34_266)          | (3_497x / 1_226x)             | (5_453x / 70x)              | (-3_497x / -1_226x)           | (2x / -18x)                   | (-5_453x / -70x)            | (-2x / 18x)                   |
| null_init_stack: 100      | (18_838_981 / 19_243_135)       | (3_716 / 5_099)          | (1_803 / 35_706)        | (5_070x / 3_774x)             | (10_449x / 539x)            | (-5_070x / -3_774x)           | (2x / -7x)                    | (-10_449x / -539x)          | (-2x / 7x)                    |
| null_init_heap: 1         | (400_365 / 806_163)             | (3_351 / 4_734)          | (2_802 / 36_726)        | (120x / 171x)                 | (145x / 22x)                | (-120x / -171x)               | (1x / -8x)                    | (-145x / -22x)              | (-1x / 8x)                    |
| null_init_heap: 10        | (2_672_253 / 3_080_750)         | (33_119 / 34_502)        | (24_676 / 58_457)       | (81x / 89x)                   | (109x / 53x)                | (-81x / -89x)                 | (1x / -2x)                    | (-109x / -53x)              | (-1x / 2x)                    |
| null_init_heap: 100       | (26_141_210 / 26_927_088)       | (1_834_154 / 1_835_537)  | (258_180 / 292_472)     | (15x / 15x)                   | (102x / 92x)                | (-15x / -15x)                 | (7x / 6x)                     | (-102x / -92x)              | (-7x / -6x)                   |
| opt_init_stack: 1         | (331_477 / 734_040)             | (267 / 1_650)            | (219 / 34_553)          | (1_241x / 445x)               | (1_514x / 21x)              | (-1_241x / -445x)             | (1x / -21x)                   | (-1_514x / -21x)            | (-1x / 21x)                   |
| opt_init_stack: 10        | (1_974_883 / 2_379_963)         | (646 / 2_029)            | (363 / 34_697)          | (3_057x / 1_173x)             | (5_440x / 69x)              | (-3_057x / -1_173x)           | (2x / -17x)                   | (-5_440x / -69x)            | (-2x / 17x)                   |
| opt_init_stack: 100       | (18_487_014 / 18_894_526)       | (4_516 / 5_899)          | (1_803 / 36_137)        | (4_094x / 3_203x)             | (10_253x / 523x)            | (-4_094x / -3_203x)           | (3x / -6x)                    | (-10_253x / -523x)          | (-3x / 6x)                    |
| opt_init_heap: 1          | (399_238 / 1_198_259)           | (3_377 / 4_760)          | (2_820 / 37_159)        | (118x / 244x)                 | (144x / 32x)                | (-118x / -244x)               | (1x / -8x)                    | (-144x / -32x)              | (-1x / 8x)                    |
| opt_init_heap: 10         | (2_660_238 / 3_063_588)         | (33_253 / 34_636)        | (21_468 / 55_821)       | (80x / 89x)                   | (127x / 55x)                | (-80x / -89x)                 | (1x / -2x)                    | (-127x / -55x)              | (-1x / 2x)                    |
| opt_init_heap: 100        | (26_069_132 / 26_475_003)       | (1_835_486 / 1_836_869)  | (215_734 / 250_089)     | (15x / 15x)                   | (123x / 107x)               | (-15x / -15x)                 | (8x / 7x)                     | (-123x / -107x)             | (-8x / -7x)                   |
| principal_init_stack: 1   | (49_695_726 / 50_498_314)       | (6_496 / 7_879)          | (21_497 / 55_831)       | (7_650x / 6_409x)             | (2_312x / 904x)             | (-7_650x / -6_409x)           | (-3x / -7x)                   | (-2_312x / -904x)           | (3x / 7x)                     |
| principal_init_stack: 10  | (329_708_888 / 330_113_607)     | (42_691 / 44_074)        | (143_519 / 177_853)     | (7_723x / 7_490x)             | (2_297x / 1_856x)           | (-7_723x / -7_490x)           | (-3x / -4x)                   | (-2_297x / -1_856x)         | (3x / 4x)                     |
| principal_init_stack: 100 | (3_297_027_262 / 3_297_432_943) | (424_966 / 426_349)      | (1_433_354 / 1_467_688) | (7_758x / 7_734x)             | (2_300x / 2_247x)           | (-7_758x / -7_734x)           | (-3x / -3x)                   | (-2_300x / -2_247x)         | (3x / 3x)                     |
| principal_init_heap: 1    | (50_178_095 / 50_582_849)       | (9_606 / 10_989)         | (25_136 / 59_475)       | (5_224x / 4_603x)             | (1_997x / 850x)             | (-5_224x / -4_603x)           | (-3x / -5x)                   | (-1_997x / -850x)           | (3x / 5x)                     |
| principal_init_heap: 10   | (330_635_793 / 331_040_910)     | (75_351 / 76_734)        | (171_756 / 205_721)     | (4_389x / 4_315x)             | (1_926x / 1_610x)           | (-4_389x / -4_315x)           | (-2x / -3x)                   | (-1_926x / -1_610x)         | (2x / 3x)                     |
| principal_init_heap: 100  | (3_302_631_760 / 3_303_036_421) | (2_256_468 / 2_257_851)  | (1_721_700 / 1_755_624) | (1_492x / 1_491x)             | (1_919x / 1_882x)           | (-1_492x / -1_491x)           | (1x / 1x)                     | (-1_919x / -1_882x)         | (-1x / -1x)                   |
| record_init_stack: 1      | (51_435_646 / 51_842_080)       | (6_980 / 8_363)          | (22_843 / 56_750)       | (7_369x / 6_199x)             | (2_252x / 914x)             | (-7_369x / -6_199x)           | (-3x / -7x)                   | (-2_252x / -914x)           | (3x / 7x)                     |
| record_init_stack: 10     | (344_365_235 / 344_771_774)     | (47_461 / 48_844)        | (153_894 / 187_801)     | (7_256x / 7_059x)             | (2_238x / 1_836x)           | (-7_256x / -7_059x)           | (-3x / -4x)                   | (-2_238x / -1_836x)         | (3x / 4x)                     |
| record_init_stack: 100    | (3_439_495_038 / 3_439_900_563) | (472_666 / 474_049)      | (1_537_104 / 1_571_011) | (7_277x / 7_256x)             | (2_238x / 2_190x)           | (-7_277x / -7_256x)           | (-3x / -3x)                   | (-2_238x / -2_190x)         | (3x / 3x)                     |
| record_init_heap: 1       | (51_525_643 / 51_933_388)       | (10_090 / 11_473)        | (25_686 / 59_610)       | (5_109x / 4_528x)             | (2_006x / 871x)             | (-5_109x / -4_528x)           | (-3x / -5x)                   | (-2_006x / -871x)           | (3x / 5x)                     |
| record_init_heap: 10      | (344_375_153 / 344_781_311)     | (80_121 / 81_504)        | (187_863 / 222_772)     | (4_299x / 4_231x)             | (1_834x / 1_548x)           | (-4_299x / -4_231x)           | (-2x / -3x)                   | (-1_834x / -1_548x)         | (2x / 3x)                     |
| record_init_heap: 100     | (3_436_270_318 / 3_436_675_836) | (2_304_168 / 2_305_551)  | (1_909_179 / 1_943_699) | (1_519x / 1_518x)             | (1_801x / 1_769x)           | (-1_519x / -1_518x)           | (1x / 1x)                     | (-1_801x / -1_769x)         | (-1x / -1x)                   |
| text_init_stack: 1        | (332_388 / 740_678)             | (251 / 1_634)            | (570 / 34_923)          | (1_324x / 453x)               | (583x / 21x)                | (-1_324x / -453x)             | (-2x / -21x)                  | (-583x / -21x)              | (2x / 21x)                    |
| text_init_stack: 10       | (1_978_957 / 2_386_888)         | (566 / 1_949)            | (2_214 / 36_567)        | (3_496x / 1_225x)             | (894x / 65x)                | (-3_496x / -1_225x)           | (-4x / -19x)                  | (-894x / -65x)              | (4x / 19x)                    |
| text_init_stack: 100      | (18_499_815 / 18_907_728)       | (3_716 / 5_099)          | (20_304 / 54_657)       | (4_978x / 3_708x)             | (911x / 346x)               | (-4_978x / -3_708x)           | (-5x / -11x)                  | (-911x / -346x)             | (5x / 11x)                    |
| text_init_heap: 1         | (401_476 / 809_788)             | (3_361 / 4_744)          | (3_296 / 38_009)        | (120x / 171x)                 | (122x / 21x)                | (-120x / -171x)               | (0x / -8x)                    | (-122x / -21x)              | (-0x / 8x)                    |
| text_init_heap: 10        | (2_678_667 / 3_086_982)         | (33_226 / 34_609)        | (25_633 / 60_393)       | (81x / 89x)                   | (105x / 51x)                | (-81x / -89x)                 | (1x / -2x)                    | (-105x / -51x)              | (-1x / 2x)                    |
| text_init_heap: 100       | (26_749_942 / 27_150_806)       | (1_835_218 / 1_836_601)  | (239_751 / 274_243)     | (15x / 15x)                   | (113x / 100x)               | (-15x / -15x)                 | (8x / 7x)                     | (-113x / -100x)             | (-8x / -7x)                   |
| variant_init_stack: 1     | (380_470 / 780_939)             | (304 / 1_687)            | (219 / 34_452)          | (1_252x / 463x)               | (1_737x / 23x)              | (-1_252x / -463x)             | (1x / -20x)                   | (-1_737x / -23x)            | (-1x / 20x)                   |
| variant_init_stack: 10    | (2_466_084 / 2_869_220)         | (1_096 / 2_479)          | (363 / 34_596)          | (2_250x / 1_157x)             | (6_794x / 83x)              | (-2_250x / -1_157x)           | (3x / -14x)                   | (-6_794x / -83x)            | (-3x / 14x)                   |
| variant_init_stack: 100   | (24_001_210 / 24_406_664)       | (9_016 / 10_399)         | (1_803 / 36_036)        | (2_662x / 2_347x)             | (13_312x / 677x)            | (-2_662x / -2_347x)           | (5x / -3x)                    | (-13_312x / -677x)          | (-5x / 3x)                    |
| variant_init_heap: 1      | (452_823 / 860_262)             | (3_414 / 4_797)          | (2_371 / 37_334)        | (133x / 180x)                 | (195x / 23x)                | (-133x / -180x)               | (1x / -8x)                    | (-195x / -23x)              | (-1x / 8x)                    |
| variant_init_heap: 10     | (3_774_947 / 4_182_172)         | (33_756 / 35_139)        | (26_215 / 61_079)       | (112x / 119x)                 | (146x / 69x)                | (-112x / -119x)               | (1x / -2x)                    | (-146x / -69x)              | (-1x / 2x)                    |
| variant_init_heap: 100    | (32_638_510 / 33_042_691)       | (1_840_518 / 1_841_901)  | (262_013 / 296_020)     | (18x / 18x)                   | (126x / 112x)               | (-18x / -18x)                 | (7x / 6x)                     | (-126x / -112x)             | (-7x / -6x)                   |
| vec_init_stack: 1         | (801_208 / 1_206_649)           | (251 / 1_634)            | (1_041 / 34_934)        | (3_192x / 738x)               | (770x / 35x)                | (-3_192x / -738x)             | (-4x / -21x)                  | (-770x / -35x)              | (4x / 21x)                    |
| vec_init_stack: 10        | (4_478_190 / 4_885_180)         | (566 / 1_949)            | (4_569 / 38_462)        | (7_912x / 2_507x)             | (980x / 127x)               | (-7_912x / -2_507x)           | (-8x / -20x)                  | (-980x / -127x)             | (8x / 20x)                    |
| vec_init_stack: 100       | (44_116_905 / 44_521_820)       | (3_716 / 5_099)          | (43_854 / 77_747)       | (11_872x / 8_731x)            | (1_006x / 573x)             | (-11_872x / -8_731x)          | (-12x / -15x)                 | (-1_006x / -573x)           | (12x / 15x)                   |
| vec_init_heap: 1          | (870_181 / 1_276_858)           | (3_361 / 4_744)          | (3_528 / 37_254)        | (259x / 269x)                 | (249x / 34x)                | (-259x / -269x)               | (-1x / -8x)                   | (-249x / -34x)              | (1x / 8x)                     |
| vec_init_heap: 10         | (5_874_947 / 6_279_492)         | (33_226 / 34_609)        | (26_005 / 59_540)       | (177x / 181x)                 | (233x / 106x)               | (-177x / -181x)               | (1x / -2x)                    | (-233x / -106x)             | (-1x / 2x)                    |
| vec_init_heap: 100        | (52_692_992 / 53_096_714)       | (1_835_218 / 1_836_601)  | (242_903 / 276_406)     | (30x / 30x)                   | (220x / 194x)               | (-30x / -30x)                 | (8x / 7x)                     | (-220x / -194x)             | (-8x / -7x)                   |
